@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1998-2016. All Rights Reserved.
+%% Copyright Ericsson AB 1998-2018. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -34,9 +34,10 @@
 
 -include("mnesia.hrl").
 
+%% Local function in order to avoid external function call
 val(Var) ->
-    case ?catch_val(Var) of
-	{'EXIT', _} -> mnesia_lib:other_val(Var);
+    case ?catch_val_and_stack(Var) of
+	{'EXIT', Stacktrace} -> mnesia_lib:other_val(Var, Stacktrace);
 	Value -> Value
     end.
 
@@ -46,7 +47,7 @@ val(Var) ->
 disc_load_table(Tab, Reason) ->
     Storage =  val({Tab, storage_type}),
     Type = val({Tab, setorbag}),
-    dbg_out("Getting table ~p (~p) from disc: ~p~n",
+    dbg_out("Getting table ~tp (~p) from disc: ~tp~n",
 	    [Tab, Storage, Reason]),
     ?eval_debug_fun({?MODULE, do_get_disc_copy},
 		    [{tab, Tab},
@@ -56,7 +57,7 @@ disc_load_table(Tab, Reason) ->
     do_get_disc_copy2(Tab, Reason, Storage, Type).
 
 do_get_disc_copy2(Tab, _Reason, Storage, _Type) when Storage == unknown ->
-    verbose("Local table copy of ~p has recently been deleted, ignored.~n",
+    verbose("Local table copy of ~tp has recently been deleted, ignored.~n",
 	    [Tab]),
     {not_loaded, storage_unknown};
 do_get_disc_copy2(Tab, Reason, Storage, Type) when Storage == disc_copies ->
@@ -199,20 +200,20 @@ net_load_table(Tab, Reason, Ns, _Cs) ->
     try_net_load_table(Tab, Reason, Ns, val({Tab, cstruct})).
 
 try_net_load_table(Tab, _Reason, [], _Cs) ->
-    verbose("Copy failed. No active replicas of ~p are available.~n", [Tab]),
+    verbose("Copy failed. No active replicas of ~tp are available.~n", [Tab]),
     {not_loaded, none_active};
 try_net_load_table(Tab, Reason, Ns, Cs) ->
     Storage = mnesia_lib:cs_to_storage_type(node(), Cs),
     do_get_network_copy(Tab, Reason, Ns, Storage, Cs).
 
 do_get_network_copy(Tab, _Reason, _Ns, unknown, _Cs) ->
-    verbose("Local table copy of ~p has recently been deleted, ignored.~n", [Tab]),
+    verbose("Local table copy of ~tp has recently been deleted, ignored.~n", [Tab]),
     {not_loaded, storage_unknown};
 do_get_network_copy(Tab, Reason, Ns, Storage, Cs) ->
     [Node | Tail] = Ns,
     case lists:member(Node,val({current, db_nodes})) of
 	true ->
-	    dbg_out("Getting table ~p (~p) from node ~p: ~p~n",
+	    dbg_out("Getting table ~tp (~p) from node ~p: ~tp~n",
 		    [Tab, Storage, Node, Reason]),
 	    ?eval_debug_fun({?MODULE, do_get_network_copy},
 			    [{tab, Tab}, {reason, Reason},
@@ -222,7 +223,7 @@ do_get_network_copy(Tab, Reason, Ns, Storage, Cs) ->
 		    set({Tab, load_node}, Node),
 		    set({Tab, load_reason}, Reason),
 		    mnesia_controller:i_have_tab(Tab),
-		    dbg_out("Table ~p copied from ~p to ~p~n", [Tab, Node, node()]),
+		    dbg_out("Table ~tp copied from ~p to ~p~n", [Tab, Node, node()]),
 		    {loaded, ok};
 		Err = {error, _} when element(1, Reason) == dumper ->
 		    {not_loaded,Err};
@@ -286,12 +287,12 @@ init_receiver(Node, Tab,Storage,Cs,Reason) ->
 		  element(1,Reason) == dumper ->
 		{error,Result};
 	    {atomic, {error,Result}} ->
-		fatal("Cannot create table ~p: ~p~n",
+		fatal("Cannot create table ~tp: ~tp~n",
 		      [[Tab, Storage], Result]);
 	    {atomic,  Result} -> Result;
 	    {aborted, nomore} -> restart;
 	    {aborted, _Reas} ->
-		verbose("Receiver failed on ~p from ~p:~nReason: ~p~n",
+		verbose("Receiver failed on ~tp from ~p:~nReason: ~tp~n",
 			[Tab,Node,_Reas]),
 		down  %% either this node or sender is dying
 	end,
@@ -313,7 +314,7 @@ start_remote_sender(Node,Tab,Storage) ->
 	    {SenderPid, TabSize, DetsData};
 	%% Protocol conversion hack
 	{copier_done, Node} ->
-	    verbose("Sender of table ~p crashed on node ~p ~n", [Tab, Node]),
+	    verbose("Sender of table ~tp crashed on node ~p ~n", [Tab, Node]),
 	    down(Tab, Storage)
     end.
 
@@ -342,9 +343,12 @@ spawned_receiver(ReplyTo,Tab,Storage,Cs, SenderPid,TabSize,DetsData, Init) ->
     Done = do_init_table(Tab,Storage,Cs,
 			 SenderPid,TabSize,DetsData,
 			 ReplyTo, Init),
-    ReplyTo ! {self(),Done},
-    unlink(ReplyTo),
-    unlink(whereis(mnesia_controller)),
+    try
+        ReplyTo ! {self(),Done},
+        unlink(ReplyTo),
+        unlink(whereis(mnesia_controller))
+    catch _:_ -> ok %% avoid error reports when stopping down mnesia
+    end,
     exit(normal).
 
 wait_on_load_complete(Pid) ->
@@ -371,7 +375,7 @@ do_init_table(Tab,Storage,Cs,SenderPid,
 		    tab_receiver(Node,Tab,Storage,Cs,OrigTabRec);
 		Reason ->
 		    Msg = "[d]ets:init table failed",
-		    verbose("~s: ~p: ~p~n", [Msg, Tab, Reason]),
+		    verbose("~ts: ~tp: ~tp~n", [Msg, Tab, Reason]),
 		    down(Tab, Storage)
 	    end;
 	Error ->
@@ -429,7 +433,7 @@ tab_receiver(Node, Tab, Storage, Cs, OrigTabRec) ->
 
 	%% Protocol conversion hack
 	{copier_done, Node} ->
-	    verbose("Sender of table ~p crashed on node ~p ~n", [Tab, Node]),
+	    verbose("Sender of table ~tp crashed on node ~p ~n", [Tab, Node]),
 	    down(Tab, Storage);
 
 	{'EXIT', Pid, Reason} ->
@@ -487,7 +491,7 @@ ext_load_table(Mod, Alias, Tab, Reason) ->
 ext_init_table(Action, Alias, Mod, Tab, Fun, State, Sender) ->
     case Fun(Action) of
 	{copier_done, Node} ->
-	    verbose("Receiver of table ~p crashed on ~p (more)~n", [Tab, Node]),
+	    verbose("Receiver of table ~tp crashed on ~p (more)~n", [Tab, Node]),
 	    down(Tab, {ext,Alias,Mod});
 	{Data, NewFun} ->
 	    case Mod:receive_data(Data, Alias, Tab, Sender, State) of
@@ -532,7 +536,7 @@ init_table(Tab, _, Fun, _DetsInfo,_) ->
     try
 	true = ets:init_table(Tab, Fun),
 	ok
-    catch _:Else -> {Else, erlang:get_stacktrace()}
+    catch _:Else:Stacktrace -> {Else, Stacktrace}
     end.
 
 
@@ -550,7 +554,7 @@ finish_copy(Storage,Tab,Cs,SenderPid,DatBin,OrigTabRec) ->
 	    ok;
 	{error, Reason} ->
 	    Msg = "Failed to handle last",
-	    verbose("~s: ~p: ~p~n", [Msg, Tab, Reason]),
+	    verbose("~ts: ~tp: ~tp~n", [Msg, Tab, Reason]),
 	    down(Tab, Storage)
     end.
 
@@ -774,9 +778,9 @@ do_send_table(Pid, Tab, Storage, RemoteS) ->
         throw:receiver_died ->
             cleanup_tab_copier(Pid, Storage, Tab),
             ok;
-        error:Reason -> %% Prepare failed
+        error:Reason:Stacktrace -> %% Prepare failed
             cleanup_tab_copier(Pid, Storage, Tab),
-            {error, {tab_copier, Tab, {Reason, erlang:get_stacktrace()}}}
+            {error, {tab_copier, Tab, {Reason, Stacktrace}}}
     after
         unlink(whereis(mnesia_tm))
     end.
@@ -856,7 +860,7 @@ send_more(Pid, N, Chunk, DataState, Tab, Storage) ->
 	    send_more(Pid, 1, NewChunk, Init(), Tab, Storage);
 
 	{copier_done, Node} when Node == node(Pid)->
-	    verbose("Receiver of table ~p crashed on ~p (more)~n", [Tab, Node]),
+	    verbose("Receiver of table ~tp crashed on ~p (more)~n", [Tab, Node]),
 	    throw(receiver_died)
     end.
 
@@ -916,9 +920,15 @@ send_packet(_N, _Pid, _Chunk, DataState) ->
 finish_copy(Pid, Tab, Storage, RemoteS, NeedLock) ->
     RecNode = node(Pid),
     DatBin = dat2bin(Tab, Storage, RemoteS),
+    Node = node(Pid),
     Trans =
 	fun() ->
 		NeedLock andalso mnesia:read_lock_table(Tab),
+                %% Check that receiver is still alive
+                receive {copier_done, Node} ->
+                        throw(receiver_died)
+                after 0 -> ok
+                end,
 		A = val({Tab, access_mode}),
 		mnesia_controller:sync_and_block_table_whereabouts(Tab, RecNode, RemoteS, A),
 		cleanup_tab_copier(Pid, Storage, Tab),
@@ -927,8 +937,8 @@ finish_copy(Pid, Tab, Storage, RemoteS, NeedLock) ->
 		receive
 		    {Pid, no_more} -> % Dont bother about the spurious 'more' message
 			no_more;
-		    {copier_done, Node} when Node == node(Pid)->
-			verbose("Tab receiver ~p crashed (more): ~p~n", [Tab, Node]),
+		    {copier_done, Node} ->
+			verbose("Tab receiver ~tp crashed (more): ~p~n", [Tab, Node]),
 			receiver_died
 		end
 	end,
